import java.util.TimeZone;
import java.util.Date;
import java.time.Duration;
import java.util.Arrays;
import java.io.*;

/**
Quick script to list all the TimeZones the JVM knows about.
Mainly to show life is never as simple as you want it to be.
*/
public class ListTimeZones {

	private static String tzStr = "TZ code: %s, TZ name: %s, current UTC offset is %s, DST is: %s%s\n";

    public static void main(String[] args) throws Exception {
		// ensure we get a unicode capable output stream
		PrintWriter consoleOut =
			new PrintWriter(new OutputStreamWriter(System.out, "UTF-8"));

		String[] tzs = TimeZone.getAvailableIDs();
		Arrays.sort(tzs);

		Date now = new Date();
		long nowl = now.getTime(); // boo for Java's datetime handling
		consoleOut.println("Current datetime: "+now.toString());
		for (String tz : tzs) {
			TimeZone t = TimeZone.getTimeZone(tz);
			Duration dst = Duration.ofMillis(t.getDSTSavings());
			Duration off = Duration.ofMillis(t.getOffset(nowl));
			consoleOut.printf(tzStr,
				tz,
				t.getDisplayName(),
				off.toString().substring(2),
				dst.toString().substring(2),
				(t.inDaylightTime(now) ? ", ACTIVE" : "")
			);
		}

		consoleOut.println("TimeZone count: "+tzs.length);
		consoleOut.close();
	}
}
